@@ -1,7 +1,7 @@
 /*
  *  Copyright 2008-2021 NVIDIA Corporation
  *  Copyright 2013 Filipe RNC Maia
- *  Modifications Copyright© 2019 Advanced Micro Devices, Inc. All rights reserved. 
+ *  Modifications Copyright© 2019-2025 Advanced Micro Devices, Inc. All rights reserved.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -44,7 +44,6 @@
 
 /* adapted from FreeBSDs msun:*/
 
-
 #pragma once
 
 #include <thrust/detail/config.h>
@@ -53,23 +52,24 @@
 #include <thrust/detail/complex/math_private.h>
 
 THRUST_NAMESPACE_BEGIN
-namespace detail{
-namespace complex{
+namespace detail
+{
+namespace complex
+{
 
 using thrust::complex;
 
 /* round down to 18 = 54/3 bits */
-__host__ __device__ inline
-double trim(double x){
+THRUST_HOST_DEVICE inline double trim(double x)
+{
   uint32_t hi;
   get_high_word(hi, x);
-  insert_words(x, hi &0xfffffff8, 0);
+  insert_words(x, hi & 0xfffffff8, 0);
   return x;
 }
 
-__host__ __device__ inline
-complex<double> clog(const complex<double>& z){
-
+THRUST_HOST_DEVICE inline complex<double> clog(const complex<double>& z)
+{
   // Adapted from FreeBSDs msun
   double x, y;
   double ax, ay;
@@ -82,14 +82,16 @@ complex<double> clog(const complex<double>& z){
   y = z.imag();
 
   /* Handle NaNs using the general formula to mix them right. */
-  if (x != x || y != y){
+  if (x != x || y != y)
+  {
     return (complex<double>(log(norm(z)), atan2(y, x)));
   }
 
   ax = std::abs(x);
   ay = std::abs(y);
-  if (ax < ay) {
-    t = ax;
+  if (ax < ay)
+  {
+    t  = ax;
     ax = ay;
     ay = t;
   }
@@ -104,11 +106,14 @@ complex<double> clog(const complex<double>& z){
   //    if (ay > 5e307){
   // For high values of ay -> hypotf(DBL_MAX,ay) = inf
   // We expect that for values at or below ay = 5e307 this should not happen
-  if (ay > 5e307){
+  if (ay > 5e307)
+  {
     return (complex<double>(log(hypot(x / e, y / e)) + 1.0, atan2(y, x)));
   }
-  if (ax == 1.) {
-    if (ay < 1e-150){
+  if (ax == 1.)
+  {
+    if (ay < 1e-150)
+    {
       return (complex<double>((ay * 0.5) * ay, atan2(y, x)));
     }
     return (complex<double>(log1p(ay * ay) * 0.5, atan2(y, x)));
@@ -118,7 +123,8 @@ complex<double> clog(const complex<double>& z){
    * Because atan2 and hypot conform to C99, this also covers all the
    * edge cases when x or y are 0 or infinite.
    */
-  if (ax < 1e-50 || ay < 1e-50 || ax > 1e50 || ay > 1e50){
+  if (ax < 1e-50 || ay < 1e-50 || ax > 1e50 || ay > 1e50)
+  {
     return (complex<double>(log(hypot(x, y)), atan2(y, x)));
   }
 
@@ -129,12 +135,14 @@ complex<double> clog(const complex<double>& z){
 
   /* Some easy cases. */
 
-  if (ax >= 1.0){
-    return (complex<double>(log1p((ax-1)*(ax+1) + ay*ay) * 0.5, atan2(y, x)));
+  if (ax >= 1.0)
+  {
+    return (complex<double>(log1p((ax - 1) * (ax + 1) + ay * ay) * 0.5, atan2(y, x)));
   }
 
-  if (ax*ax + ay*ay <= 0.7){
-    return (complex<double>(log(ax*ax + ay*ay) * 0.5, atan2(y, x)));
+  if (ax * ax + ay * ay <= 0.7)
+  {
+    return (complex<double>(log(ax * ax + ay * ay) * 0.5, atan2(y, x)));
   }
 
   /*
@@ -142,45 +150,48 @@ complex<double> clog(const complex<double>& z){
    * moderately close to 1.
    */
 
-
   x0 = trim(ax);
-  ax = ax-x0;
+  ax = ax - x0;
   x1 = trim(ax);
-  x2 = ax-x1;
+  x2 = ax - x1;
   y0 = trim(ay);
-  ay = ay-y0;
+  ay = ay - y0;
   y1 = trim(ay);
-  y2 = ay-y1;
+  y2 = ay - y1;
 
-  val[0] = x0*x0;
-  val[1] = y0*y0;
-  val[2] = 2*x0*x1;
-  val[3] = 2*y0*y1;
-  val[4] = x1*x1;
-  val[5] = y1*y1;
-  val[6] = 2*x0*x2;
-  val[7] = 2*y0*y2;
-  val[8] = 2*x1*x2;
-  val[9] = 2*y1*y2;
-  val[10] = x2*x2;
-  val[11] = y2*y2;
+  val[0]  = x0 * x0;
+  val[1]  = y0 * y0;
+  val[2]  = 2 * x0 * x1;
+  val[3]  = 2 * y0 * y1;
+  val[4]  = x1 * x1;
+  val[5]  = y1 * y1;
+  val[6]  = 2 * x0 * x2;
+  val[7]  = 2 * y0 * y2;
+  val[8]  = 2 * x1 * x2;
+  val[9]  = 2 * y1 * y2;
+  val[10] = x2 * x2;
+  val[11] = y2 * y2;
 
   /* Bubble sort. */
 
-  do {
+  do
+  {
     sorted = 1;
-    for (i=0;i<11;i++) {
-      if (val[i] < val[i+1]) {
-	sorted = 0;
-	t = val[i];
-	val[i] = val[i+1];
-	val[i+1] = t;
+    for (i = 0; i < 11; i++)
+    {
+      if (val[i] < val[i + 1])
+      {
+        sorted     = 0;
+        t          = val[i];
+        val[i]     = val[i + 1];
+        val[i + 1] = t;
       }
     }
   } while (!sorted);
 
   hm1 = -1;
-  for (i=0;i<12;i++){
+  for (i = 0; i < 12; i++)
+  {
     hm1 += val[i];
   }
   return (complex<double>(0.5 * log1p(hm1), atan2(y, x)));
@@ -191,24 +202,23 @@ complex<double> clog(const complex<double>& z){
 } // namespace detail
 
 template <typename ValueType>
-__host__ __device__
-inline complex<ValueType> log(const complex<ValueType>& z){
-  return complex<ValueType>(log(thrust::abs(z)),thrust::arg(z));
+THRUST_HOST_DEVICE inline complex<ValueType> log(const complex<ValueType>& z)
+{
+  return complex<ValueType>(std::log(thrust::abs(z)), thrust::arg(z));
 }
 
 template <>
-__host__ __device__
-inline complex<double> log(const complex<double>& z){
+THRUST_HOST_DEVICE inline complex<double> log(const complex<double>& z)
+{
   return detail::complex::clog(z);
 }
 
 template <typename ValueType>
-__host__ __device__
-inline complex<ValueType> log10(const complex<ValueType>& z){
+THRUST_HOST_DEVICE inline complex<ValueType> log10(const complex<ValueType>& z)
+{
   // Using the explicit literal prevents compile time warnings in
   // devices that don't support doubles
-  return thrust::log(z)/ValueType(2.30258509299404568402);
+  return thrust::log(z) / ValueType(2.30258509299404568402);
 }
 
 THRUST_NAMESPACE_END
-
